#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <dos.h>
#include "104i16.h"

#ifdef __cplusplus
    // if it's a C++ program, ISRs take on ... argument
    #define __CPPARGS ...
#else
    // otherwise nothing
    #define __CPPARGS
#endif

typedef enum {FALSE,TRUE} boolean;

unsigned IRQ=5;
boolean passed,isr_flag;
unsigned int base;

void interrupt far (*oldisr)(__CPPARGS);  //global to store old handler pointer


void printbits(char foo)
{ int i;
  for(i=7;i>=0;i--){
    if (foo & (1<<i))
      putch('1');
    else
      putch('0');
  }
}

void print16bits(unsigned foo)
{ int i;
  for(i=15;i>=0;i--){
    if (foo & (1<<i))
      putch('1');
    else
      putch('0');
  }
}


void sendEOI(void)
{
   //send non-specific to master AND slave PIC
   outportb(0x20, 0x20);
   outportb(0xA0, 0x20);
}

void interrupt ISR(__CPPARGS)
{
   isr_flag = TRUE;
//   outportb(base+2,0x00);//disable IRQ
//   inportb(base+2);//enable IRQ
   outportb(base+1,0);
   sendEOI();
}


unsigned char initirq(char IRQnumber,void interrupt (*ISR)())
{
   unsigned char intmask, oldmask;

   if (IRQnumber <= 7) {
      oldmask = inportb(0x21);
      oldisr = getvect(IRQnumber + 8);
      setvect(IRQnumber + 8,ISR);
      intmask = oldmask & (~(1 << IRQnumber));
      outportb(0x21,intmask);
   } else {
      oldmask = inportb(0xA1);
      oldisr = getvect(IRQnumber + 0x70 - 8);
      setvect(IRQnumber - 8 + 0x70, ISR);
      intmask = oldmask & (~(1 << (IRQnumber - 8)));
      outportb(0xA1, intmask);
   }
   return(oldmask);
}

void restoreirq(char IRQnumber)
{
   unsigned char intmask;

   if (IRQnumber <=7 ){
      intmask = inportb(0x21);
      intmask |= (1 << IRQnumber);
      setvect(IRQnumber + 8,oldisr);
      outportb(0x21, intmask);
   } else {
      intmask = inportb(0xA1);
      intmask |= (1 << (IRQnumber - 8));
      setvect(IRQnumber - 8 + 0x70, oldisr);
      outportb(0xA1, intmask);
   }
}

void IRQTest(void)
{
   unsigned int i;
   int direction=1;
   unsigned char out=0;
   unsigned char outstore=0xff;
   isr_flag = FALSE;
   outportb(base+0,0xFF);
   outportb(base+4, 0xFF);
   outportb(base+1,0);
   delay(100);

   i = 0;
   out=0;
   clrscr();
   printf("Install IRQ jumper on 5.\n");
   printf("All 16 Relays will be turned ON together, then OFF individualy by the program\n");
   printf("to test if an input COS interrupt was generated.\n");
   printf("Press any key to begin the IRQ test.\n");
   getch();

   initirq(IRQ, ISR);
   inportb(base+2);//enable IRQs
   //from this point on, any IRQs will enter ISR code.

   do{
     if (!i)
     {//beginning of loop, i==0
       outstore=~outstore;//0<-->FF
       out=outstore;
       direction*=-1;
     }
     if (i==8) out=outstore;
     isr_flag = FALSE;
     if (direction==-1)
       out |= (1 << (i%8));
     else
       out &=(~(1<<(i%8)));

     outportb(base+((i<8)?0:4),out);
     delay(300);
     gotoxy(10,8+i+((i<8)?0:1));
     if (!isr_flag)
       printf("Bit %d Interrupt Failed. value=%02x", i,out);
     else
       printf("Bit %d Interrupt Passed. value=%02x", i,out);
     i++;
     i%=16;
        outportb(base+1,0);
   }while(!kbhit());
   getch();
   restoreirq(IRQ);
   clrscr();
}

void RelayTest(void)
{
   char ch, *valstring;
   boolean passed, *funct;
   unsigned char outbyte, inbyte;
   int count, int_mask, loop, i;
   int store1[16], store2[16], errorblk[16];

   printf("Verify FLT jumpers are OFF, then hit any key to continue.\n");
   getch();

   for (i = 0; i <16; i++) {
      outbyte=1<<(i%8);
      for(count=0,loop = 0; loop < 5; loop++) {
         // Write to base address + 0 or + 4.
         outportb(base+(i>7?4:0), outbyte);
         do {
            count++;
         // Read from base address + 1. Increment counter and read from the card until the data read matches the data sent out.
            inbyte = inportb(base + 1 + (i>7?4:0));
         } while (!(count < 0) && inbyte != outbyte);
                       /* Safety feature which ends
                          the loop when a negative
                          number is reached through
                          overflow. The values should
                          never legally get that
                          high. */
         outbyte^=1<<(i%8);
      }  // end for loop
      count /= 5; /* Take the average. */
      store1[i] = count;
      delay(5);
   }  // end for int_mask


   printf("Install FLT jumpers and press any key.\n\n");
   getch();


   for (i = 0; i < 16; i++) {
      count = 0;
      outbyte = 1 << (i%8);
      for (loop=0; loop < 5; loop++) {
         outportb(base+(i>7?4:0), outbyte);
         do {
            count++;
            inbyte = inportb(base+1+(i>7?4:0));
         } while (!(count < 0) && inbyte != outbyte);
         outbyte^=1<<(i%8);
       }  //  end for loop
       count /= 5; // Take the average.
       store2[i] = count;
       errorblk[i] = 0;
       delay(5);
   }  //  end for i

   for(i = 0; i < 16; i++) {
      if ((store1[i] * 4.0) > store2[i]) {
         passed = FALSE;
         errorblk[i] = 1;
      }
      else if ((store1[i] * 8.0) < store2[i]) {
         passed = FALSE;
         errorblk[i] = 2;
      }
      else
         errorblk[i] = 0;
   }  //  end for i

   printf("  Relay    No Filters    Filters\n");
   printf("  _____    __________    _______\n");
   for (i = 0; i < 16; i++) {
      gotoxy(3, i + 7);
      switch (errorblk[i]) {
         case 0 : printf("%3d      %8d    %8d\n", i, store1[i], store2[i]);
                  break;
         case 1 : printf("%3d      %8d    %8d  LOW\n", i, store1[i], store2[i]);
                  break;
         case 2 : printf("%3d      %8d    %8d  HIGH\n", i, store1[i], store2[i]);
                  break;
      }
   }
   printf(passed?"\n Test PASSED\n":"\n Test FAILED\n");
   printf("\n\nHit any key to continue with IRQ test.\n");
   getch();
}  //  end RelayTest

void InputDisplay(void)
{
   int I;

   clrscr();
   printf("Displaying TTL inputs(104-IIRO-8 only)...\n");

   I = 0;
   do
   {
      delay(300);
      gotoxy(10, 5 + I);
      printf("%02lX", inportb(base + 3) + 0L);
      I = (I + 1) % 16;
   }
   while(!kbhit());
   getch();
}

void ReadBackTest(void)
{
   printf("   104-IIRO-8 / 16 Test Program.\n");
   printf("Base Address needs to be set to 300 hex.\n");
   printf("Install test cable from P1 to P2 for IIRO-16, P1 only for IIRO-8\n");
   printf("Apply external power ( 3 to 28V )to the red wire ( Relay C contacts )\n");
   printf("Apply external ground to the black wire ( Isolated Inputs - )\n");
   printf("hit any key to start loopback test.\n\n");
   getch();

   printf("     Output                 Input    (MSB -> LSB)\n");
   printf("     ------                 -----    \n");

   gotoxy(2, 10);
   print16bits(0x0000);
   outportb(base, 0x00);
   outportb(base +4, 0x00);
   delay(10);
   printf("      ");
   print16bits((inportb(base + 5) * 256) + inportb(base + 1));
   delay(500);
   printf("\n\n");

   gotoxy(2, 12);
   print16bits(0x5555);
   outportb(base, 0x55);
   outportb(base +4, 0x55);
   delay(10);
   printf("      ");
   print16bits((inportb(base + 5) * 256) + inportb(base + 1));
   delay(500);
   printf("\n\n");

   gotoxy(2, 14);
   print16bits(0xAAAA);
   outportb(base, 0xAA);
   outportb(base +4, 0xAA);
   delay(10);
   printf("      ");
   print16bits((inportb(base + 5) * 256) + inportb(base + 1));
   delay(500);
   printf("\n\n");

   gotoxy(2, 16);
   print16bits(0xFFFF);
   outportb(base, 0xFF);
   outportb(base +4, 0xFF);
   delay(10);
   printf("      ");
   print16bits((inportb(base + 5) * 256) + inportb(base + 1));
   delay(500);
   printf("\n\n");

   printf("Press any key to continue with filter test.\n");
   getch();
   clrscr();
}  //  end Test

void main()
{
   char ch;

   clrscr();

   base=0x300;
   IRQ=5;
   outportb(base+2, 0);
   ReadBackTest();
   RelayTest();
   IRQTest();
   InputDisplay();
}


