#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <dos.h>
#include "104i8.h"

#ifdef __cplusplus
    // if it's a C++ program, ISRs take on ... argument
    #define __CPPARGS ...
#else
    // otherwise nothing
    #define __CPPARGS
#endif

typedef enum {FALSE,TRUE} boolean;

unsigned IRQ=5;
boolean passed,isr_flag;
unsigned int base;

void interrupt far (*oldisr)(__CPPARGS);  //global to store old handler pointer


void printbits(char foo)
{ int i;
  for(i=7;i>=0;i--){
    if (foo & (1<<i))
      putch('1');
    else
      putch('0');
  }
}

void sendEOI(void)
{
   //send non-specific to master AND slave PIC
   outportb(0x20, 0x20);
   outportb(0xA0, 0x20);
}

void interrupt ISR(__CPPARGS)
{
   isr_flag = TRUE;
   outportb(base+2,0x00);//disable IRQ
   inportb(base+2);//enable IRQ
   sendEOI();
}


unsigned char initirq(char IRQnumber,void interrupt (*ISR)())
{
   unsigned char intmask, oldmask;

   if (IRQnumber <= 7) {
      oldmask = inportb(0x21);
      oldisr = getvect(IRQnumber + 8);
      setvect(IRQnumber + 8,ISR);
      intmask = oldmask & (~(1 << IRQnumber));
      outportb(0x21,intmask);
   } else {
      oldmask = inportb(0xA1);
      oldisr = getvect(IRQnumber + 0x70 - 8);
      setvect(IRQnumber - 8 + 0x70, ISR);
      intmask = oldmask & (~(1 << (IRQnumber - 8)));
      outportb(0xA1, intmask);
   }
   return(oldmask);
}

void restoreirq(char IRQnumber)
{
   unsigned char intmask;

   if (IRQnumber <=7 ){
      intmask = inportb(0x21);
      intmask |= (1 << IRQnumber);
      setvect(IRQnumber + 8,oldisr);
      outportb(0x21, intmask);
   } else {
      intmask = inportb(0xA1);
      intmask |= (1 << (IRQnumber - 8));
      setvect(IRQnumber - 8 + 0x70, oldisr);
      outportb(0xA1, intmask);
   }
}

void IRQTest(void)
{
   unsigned int i;
   int direction=1;
   unsigned char out=0;
   unsigned char outstore=0xff;
   isr_flag = FALSE;

   i = 0;
   out=0;
   clrscr();
   printf("IRQ needs to be set to 5.\n");
   printf("8 bits will be written to by the program which tests\n");
   printf("each to see if an interrupt was generated.\n");
   printf("Press any key to begin writing.\n");
   getch();

   initirq(IRQ,ISR);
   inportb(base+2);//enable IRQs
   //from this point on, any IRQs will enter ISR code.
   do{
     if (!i)
     {//beginning of loop, i==0
       outstore=~outstore;//0<-->FF
       out=outstore;
       direction*=-1;
     }
     isr_flag = FALSE;
     if (direction==-1)
       out |= (1 << i);
     else
       out &=(~(1<<i));

     outportb(base+0,out);
     delay(300);
     gotoxy(10,8+i);
     if (!isr_flag)
       printf("Bit %d Interrupt Failed. value=%02x", i,out);
     else
       printf("Bit %d Interrupt Passed. value=%02x", i,out);
     i++;
     i%=8;
   }while(!kbhit());
   getch();
   restoreirq(IRQ);
   clrscr();
}

void RelayTest(void)
{
   char ch, *valstring;
   boolean passed, *funct;
   unsigned char outbyte, inbyte;
   int count, int_mask, loop, i;
   int store1[16], store2[16], errorblk[16];

   printf("First, put filters in programmable mode, then hit any key to continue.\n");
   getch();

   for (i = 0; i <8; i++) {
      outbyte=1<<i;
      for(count=0,loop = 0; loop < 5; loop++) {
         // Write to base address + 0 or + 4.
         outportb(base, outbyte);
         do {
            count++;
         // Read from base address + 1. Increment counter and read from the card until the data read matches the data sent out.
            inbyte = inportb(base + 1);
         } while (!(count < 0) && inbyte != outbyte);
                       /* Safety feature which ends
						  the loop when a negative
						  number is reached through
						  overflow. The values should
						  never legally get that
						  high. */
         outbyte^=1<<i;
      }  // end for loop
      count /= 5; /* Take the average. */
      store1[i] = count;
      delay(5);
   }  // end for int_mask


   printf("Now enable all manual filters and press any key.\n\n");
   getch();


   for (i = 0; i < 8; i++) {
      count = 0;
      outbyte = 1 << i;
      for (loop=0; loop < 5; loop++) {
         outportb(base, outbyte);
         do {
            count++;
            inbyte = inportb(base+1);
         } while (!(count < 0) && inbyte != outbyte);
         outbyte^=1<<i;
       }  //  end for loop
       count /= 5; // Take the average.
       store2[i] = count;
       errorblk[i] = 0;
       delay(5);
   }  //  end for i

   for(i = 0; i < 8; i++) {
      if ((store1[i] * 3.0) > store2[i]) {
         passed = FALSE;
         errorblk[i] = 1;
      }
      else if ((store1[i] * 6.0) < store2[i]) {
         passed = FALSE;
         errorblk[i] = 2;
      }
      else
         errorblk[i] = 0;
   }  //  end for i

   printf("  Relay    No Filters    Filters\n");
   printf("  _____    __________    _______\n");
   for (i = 0; i < 8; i++) {
      gotoxy(5, i + 13);
      switch (errorblk[i]) {
         case 0 : printf("%d        %d        %d\n", i, store1[i], store2[i]);
                  break;
         case 1 : printf("%d        %d        %d  LOW\n", i, store1[i], store2[i]);
                  break;
         case 2 : printf("%d        %d        %d  HIGH\n", i, store1[i], store2[i]);
                  break;
      }
   }
   printf(passed?"\n Test PASSED\n":"\n Test FAILED\n");
   printf("\n\nHit any key to continue with IRQ test.\n");
   getch();
}  //  end RelayTest


void ReadBackTest(void)
{
   printf("   104-IIRO-8 Test Program.\n");
   printf("Base Address needs to be set to 300 hex.\n");
   printf("Apply a +5V external power source to the relay common pin (PIN8)\n");
   printf("and a ground to input low (PINxx), disable manual filters and install\n");
   printf("an extension card. Install wrap plug on extension card and hit any\n");
   printf("key to start test.\n\n");
   getch();

   printf("     Output                 Input    (MSB -> LSB)\n");
   printf("     ------                 -----    \n");

   gotoxy(2, 10);
   printbits(0x00);
   outportb(base, 0x00);
   delay(10);
   printf("      ");
   printbits(inportb(base + 1));
   delay(500);
   printf("\n\n");

   gotoxy(2, 12);
   printbits(0x55);
   outportb(base, 0x55);
   delay(10);
   printf("      ");
   printbits(inportb(base + 1));
   delay(500);
   printf("\n\n");

   gotoxy(2, 14);
   printbits(0xAA);
   outportb(base, 0xAA);
   delay(10);
   printf("      ");
   printbits(inportb(base + 1));
   delay(500);
   printf("\n\n");

   gotoxy(2, 16);
   printbits(0xFF);
   outportb(base, 0xFF);
   delay(10);
   printf("      ");
   printbits(inportb(base + 1));
   delay(500);
   printf("\n\n");

   printf("Press any key to continue with filter test.\n");
   getch();
   clrscr();
}  //  end Test

void main()
{
   char ch;

   clrscr();
   printf("104-IIRO-8 Test program.\n\nReading back data from base+3\nPress any key to move on.");

   base=0x300;
   IRQ=5;
   while (!kbhit()){
     gotoxy(1,10);
     printf("DATA=");
     printbits(inportb(base+3));
   }
   clrscr();
   ReadBackTest();
   RelayTest();
   IRQTest();
}


